/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "logarithmicVelocityFvPatchVectorField.H"
#include "addToRunTimeSelectionTable.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

logarithmicVelocityFvPatchVectorField::logarithmicVelocityFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchVectorField(p, iF),
    Vel1_(5.5),
    Vel2_(8),
    h1_(1),
    h2_(2),
    n_(1, 0, 0),
    y_(0, 1, 0)
{}


logarithmicVelocityFvPatchVectorField::logarithmicVelocityFvPatchVectorField
(
    const logarithmicVelocityFvPatchVectorField& ptf,
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedValueFvPatchVectorField(ptf, p, iF, mapper),
    Vel1_(ptf.Vel1_),
    Vel2_(ptf.Vel2_),
    h1_(ptf.h1_),
    h2_(ptf.h2_),
    n_(ptf.n_),
    y_(ptf.y_)
{}


logarithmicVelocityFvPatchVectorField::logarithmicVelocityFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchVectorField(p, iF),
    Vel1_(readScalar(dict.lookup("Vel1"))),
    Vel2_(readScalar(dict.lookup("Vel2"))),
    h1_(readScalar(dict.lookup("h1"))),
    h2_(readScalar(dict.lookup("h2"))),
    n_(dict.lookup("n")),
    y_(dict.lookup("y"))
{
    if (mag(n_) < SMALL || mag(y_) < SMALL)
    {
        FatalErrorIn("logarithmicVelocityFvPatchVectorField(dict)")
            << "n or y given with zero size not correct"
            << abort(FatalError);
    }

    n_ /= mag(n_);
    y_ /= mag(y_);

    evaluate();
}


logarithmicVelocityFvPatchVectorField::logarithmicVelocityFvPatchVectorField
(
    const logarithmicVelocityFvPatchVectorField& fcvpvf,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchVectorField(fcvpvf, iF),
    Vel1_(fcvpvf.Vel1_),
    Vel2_(fcvpvf.Vel2_),
    h1_(fcvpvf.h1_),
    h2_(fcvpvf.h2_),
    n_(fcvpvf.n_),
    y_(fcvpvf.y_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void logarithmicVelocityFvPatchVectorField::updateCoeffs()
{
    if (updated())
    {
        return;
    }

    // Get range and orientation
    boundBox bb(patch().patch().localPoints(), true);

    vector ctr = (bb.min());
        
    //const vectorField& c = patch().Cf();
    const vectorField& c = patch().Cf();

    // Calculate local 1-D coordinate for the logarithmic profile
    scalar z0 = exp(((Vel2_*log(h1_))-(Vel1_*log(h2_)))/(Vel2_-Vel1_)); //surface roughness
    scalar k = 0.41; //von Karman constant
    scalar ufr = (Vel1_*k)/(log(h1_/z0)); //friction velocity
    scalarField coord =((c - ctr) & y_)/((bb.max() - bb.min()) & y_);
    vectorField::operator=(n_*(ufr/k)*log(((coord*((bb.max() - bb.min()) & y_))+z0)/z0));//logarithmic velocity profile
    //vectorField::operator=(n_*Vel1_*pow(coord*((bb.max() - bb.min()) & y_)/h1_,0.143));//power law velocity profile
}


// Write
void logarithmicVelocityFvPatchVectorField::write(Ostream& os) const
{
    fvPatchVectorField::write(os);
    os.writeKeyword("Vel1")
        << Vel1_ << token::END_STATEMENT << nl;
    os.writeKeyword("Vel2")
	<< Vel2_ << token::END_STATEMENT << nl;
    os.writeKeyword("h1")
	<< h1_ << token::END_STATEMENT << nl;
    os.writeKeyword("h2")
        << h2_ << token::END_STATEMENT << nl;
    os.writeKeyword("n")
        << n_ << token::END_STATEMENT << nl;
    os.writeKeyword("y")
        << y_ << token::END_STATEMENT << nl;
    writeEntry("value", os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

makePatchTypeField(fvPatchVectorField, logarithmicVelocityFvPatchVectorField);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
